#include <comic_spe.h>
#include "jacobi01_common.h"

COMIC_shared_extern();
COMIC_shared_region();

static inline void jacobi01_spe_f1();
static inline void jacobi01_spe_f2();


/***********************************************/
/* SPE main function                           */
/***********************************************/
COMIC_spe_main(
	case 1: jacobi01_spe_f1(); break;
	case 2: jacobi01_spe_f2(); break;
)


static inline void jacobi01_spe_f1()
{
	int tid = COMIC_get_spe_thread_id();

	int i, j;
	int idx;
	int m, m_factor, m_start, m_end, n;

	m = LoadInt( &COMIC_access(m) );
	n = LoadInt( &COMIC_access(n) );

	m_factor = m / COMIC_MAXSPE;
	m_start  = m_factor * tid;
	if( tid == (COMIC_MAXSPE-1) ) m_end = m;
	else                          m_end = m_start + m_factor;

	for (j=m_start; j<m_end; j++)
		for (i=0; i<n; i++) {
			idx = i+M*j;
			StoreDouble( &COMIC_access(uold[idx]), LoadDouble(&COMIC_access(u[idx])) );
		}

	COMIC_spe_barrier( 0 );
}


static inline void jacobi01_spe_f2()
{
	int tid = COMIC_get_spe_thread_id();

	short i, j;
  double omega, resid, ax, ay, b;
	double error = 0.0;
	double dbl_temp;
	int idx;
	int m, m_factor, m_start, m_end, n;

	m = LoadInt( &COMIC_access(m) );
	n = LoadInt( &COMIC_access(n) );
	omega = LoadDouble( &COMIC_access(omega) );
	ax = LoadDouble( &COMIC_access(ax) );
	ay = LoadDouble( &COMIC_access(ay) );
	b = LoadDouble( &COMIC_access(b) );
	
	m_factor = m / COMIC_MAXSPE;
	m_start = m_factor * tid + 1;
	if( tid == (COMIC_MAXSPE-1) ) m_end = m - 1;
	else                          m_end = m_start + m_factor;

	for (j=m_start; j<m_end; j++)
		for (i=1; i<n-1; i++){
			idx = i+m*j;
			dbl_temp = LoadDouble( &COMIC_access(uold[idx]) );
			resid = (
					ax * (LoadDouble( &COMIC_access(uold[idx-1]) ) + LoadDouble( &COMIC_access(uold[idx+1]) ))
					+ ay * (LoadDouble( &COMIC_access(uold[idx-m]) ) + LoadDouble( &COMIC_access(uold[idx+m]) ))
					+ b * dbl_temp - LoadDouble( &COMIC_access(f[idx]) )
					) / b;

			// update solution 
			StoreDouble( &COMIC_access(u[idx]), dbl_temp - omega * resid );

			// accumulate residual error
			error = error + resid*resid;
		}

	StoreDouble( &COMIC_access(error[tid]), error );

	COMIC_spe_barrier( 0 );
}
